package com.gree.mall.contest.logic.merchant;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.merchant.WebsitApplyAdd;
import com.gree.mall.contest.bean.merchant.WebsitApplyDetail;
import com.gree.mall.contest.bean.merchant.WebsitApplyVO;
import com.gree.mall.contest.bean.zfire.ZfireParamBean;
import com.gree.mall.contest.commonmapper.CommonMapper;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.SMSLogic;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.GoodsCategory;
import com.gree.mall.contest.plus.entity.WebsitApply;
import com.gree.mall.contest.plus.entity.WebsitApplyArea;
import com.gree.mall.contest.plus.entity.WebsitApplyLog;
import com.gree.mall.contest.plus.service.GoodsCategoryService;
import com.gree.mall.contest.plus.service.WebsitApplyAreaService;
import com.gree.mall.contest.plus.service.WebsitApplyLogService;
import com.gree.mall.contest.plus.service.WebsitApplyService;
import com.gree.mall.contest.utils.zfire.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class MerchantLogic {
    private final CommonMapper commonMapper;
    private final CommonLogic commonLogic;
    private final SMSLogic smsLogic;
    private final WebsitApplyService websitApplyService;
    private final WebsitApplyAreaService websitApplyAreaService;
    private final GoodsCategoryService goodsCategoryService;
    private final WebsitApplyLogService websitApplyLogService;

    public Boolean register(String mobile, String code) {
        smsLogic.checkSmsCode(mobile, code, "BIND");
        WebsitApply websitApply = websitApplyService.lambdaQuery().eq(WebsitApply::getMobile, mobile).last("limit 1").one();

        if (websitApply != null) {
            return false;
        }

        return true;
    }

    public WebsitApplyDetail applyMobile(String mobile) {
        WebsitApply websitApply = websitApplyService.lambdaQuery().eq(WebsitApply::getMobile, mobile).last("limit 1").one();
        WebsitApplyDetail websitApplyDetail = BeanUtil.copyProperties(websitApply, WebsitApplyDetail.class);
        List<WebsitApplyArea> list = websitApplyAreaService.lambdaQuery()
                .eq(WebsitApplyArea::getWebsitApplyId, websitApplyDetail.getWebsitApplyId()).list();
        websitApplyDetail.setWebsitApplyAreaList(list);
        return websitApplyDetail;
    }

    @Transactional(rollbackFor = Exception.class)
    public WebsitApply applyList(WebsitApplyAdd websitApplyAdd) {
        List<String> collect = goodsCategoryService.lambdaQuery()
                .in(GoodsCategory::getCategoryId, websitApplyAdd.getCategoryIds())
                .list()
                .stream().map(GoodsCategory::getName)
                .collect(Collectors.toList());

        if (CollectionUtils.isEmpty(collect))
            throw new RemoteServiceException("主营分类未选择");

        String categoryName = String.join(",", collect);
        String categoryId = String.join(",", websitApplyAdd.getCategoryIds());

        websitApplyAdd.setCategoryId(categoryId);
        websitApplyAdd.setCategoryName(categoryName);
        websitApplyAdd.setStatus("WAIT");
        websitApplyAdd.insertOrUpdate();

        websitApplyAreaService.lambdaUpdate().eq(WebsitApplyArea::getWebsitApplyId, websitApplyAdd.getWebsitApplyId()).remove();

        for (WebsitApplyArea websitApplyArea : websitApplyAdd.getWebsitApplyAreaList()) {
            websitApplyArea.setWebsitApplyId(websitApplyAdd.getWebsitApplyId());
        }
        websitApplyAreaService.saveBatch(websitApplyAdd.getWebsitApplyAreaList());

        return websitApplyAdd;
    }

    public IPage<WebsitApplyVO> pageApply(ZfireParamBean zfireParamBean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        FieldUtils.supplyParam(zfireParamBean, WebsitApplyVO.class, adminUser);
        return commonMapper.pageApply(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
    }

    public WebsitApplyDetail detail(String id) {

        WebsitApply websitApply = websitApplyService.getById(id);
        WebsitApplyDetail websitApplyDetail = BeanUtil.copyProperties(websitApply, WebsitApplyDetail.class);
        List<WebsitApplyArea> list = websitApplyAreaService.lambdaQuery().eq(WebsitApplyArea::getWebsitApplyId, websitApplyDetail.getWebsitApplyId()).list();
        websitApplyDetail.setWebsitApplyAreaList(list);

        List<WebsitApplyLog> websitApplyLogs = websitApplyLogService.lambdaQuery().eq(WebsitApplyLog::getWebsitApplyId, websitApplyDetail.getWebsitApplyId()).list();
        websitApplyDetail.setWebsitApplyLogList(websitApplyLogs);

        return websitApplyDetail;
    }

    public void updateIn(String id, String status, String remark) {
        WebsitApply websitApply = websitApplyService.getById(id);
        websitApply.setStatus(status);
        websitApply.setRemark(remark);
        websitApply.updateById();
        WebsitApplyLog websitApplyLog = new WebsitApplyLog();
        websitApplyLog.setWebsitApplyId(id);
        websitApplyLog.setStatus(status);
        websitApplyLog.setRemark(remark);
        websitApplyLog.insert();
    }
}
